/* cks_config.c - Configuration functions
 * Copyright (C) 2001-2005 CryptNET, V. Alex Brennen (VAB)
 *
 * This file is part of the CryptNET OpenPGP Public Key Server (cks).
 *
 * cks is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * cks is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "cks_config.h"


int init_config(struct cks_ctx **ctx, int load_cache)
{
	int read_conf = 1;
	int rslt = 0;


	strncpy((*ctx)->config->cks_version,"0.2.4",6);
	(*ctx)->config->db_type = D_POSTGRES;
	strncpy((*ctx)->config->bind_ip,"127.0.0.1",16);
	strncpy((*ctx)->config->bind_port,"11371",6);
	strncpy((*ctx)->config->dbsrvr_ip,"127.0.0.1",16);
	strncpy((*ctx)->config->dbsrvr_port,"5432",5);
	strncpy((*ctx)->config->dbsrvr_db,"pgpkeys",13); /* openpgp_keys */
	(*ctx)->config->listen_threads = 10;
	strncpy((*ctx)->config->sync_email,"pgp-keys@localhost",19);
	(*ctx)->config->use_cmnt = 1;
	strncpy((*ctx)->config->vrsn,"Version:  CryptNET Keyserver Version 0.2.4\n",100);
	strncpy((*ctx)->config->cmnt,"Comment:  <a href=\"http://www.clearwaterproject.org/\">http://www.clearwaterproject.org/</a>\n\n",200);
	strncpy((*ctx)->config->adm_email,"root@localhost",100);
	(*ctx)->config->acpt_v2 = 1;
	(*ctx)->config->acpt_v3 = 1;
	(*ctx)->config->max_qry = 600; /* 10 Mins. */
	(*ctx)->config->max_ret = 1000;
	(*ctx)->config->key_ret = 10;
	(*ctx)->config->dup_acpt = 10;
	(*ctx)->config->biglumber = 1;
	(*ctx)->config->aonf = 1;
	(*ctx)->config->image_pkts = 0;
	strncpy((*ctx)->config->pid_file,"/var/run/cks/cksd.pid",255);
	strncpy((*ctx)->config->err_log,"cks_error.log",200);
	strncpy((*ctx)->config->mail_err_log,"/home/pgp-keys/cks_mail_sync.log",200);
	strncpy((*ctx)->config->data_log,"cks_data.log",200);

	/* If you copy your config into the source code it will speed things up */
	if(read_conf)
	{
		rslt = read_config(ctx,load_cache);
	}

	return rslt;
}


int read_config(struct cks_ctx **ctx,int load_cache)
{
	/* Read and parse cks.conf data. */
	/* Load data into cks_config struct */
	FILE *conf_file = NULL;
	char line[201];
	int rslt = 0;


	if((conf_file = fopen(CONF,"r")) == NULL)
	{
		fprintf(stderr,"Was not able to read %s file.\n", CONF);

		return -1;
	}

	while(fgets(line,sizeof(line),conf_file) != NULL)
        {
		if( (!(memcmp(line,"#",1) == 0)) && (!(isspace(line[0]))) )
		{
			char *name = NULL;
			char *value = NULL;

			name = strtok(line," ");
			value = strtok('\0',"\0");
			value[strlen(value)-1] = '\0';

			if(memcmp(name,"bind_ip",7) == 0)
			{
				strncpy((*ctx)->config->bind_ip,value,16);
			}
			else if(memcmp(name,"bind_port",9) == 0)
			{
				strncpy((*ctx)->config->bind_port,value,10);
			}
			else if(memcmp(name,"db_type",7) == 0)
			{
				(*ctx)->config->db_type = atoi(value);
			}
			else if(memcmp(name,"dbsrvr_ip",9) == 0)
			{
				strncpy((*ctx)->config->dbsrvr_ip,value,16);
			}
			else if(memcmp(name,"dbsrvr_port",11) == 0)
			{
				strncpy((*ctx)->config->dbsrvr_port,value,10);
			}
			else if(memcmp(name,"dbsrvr_db",9) == 0)
			{
				strncpy((*ctx)->config->dbsrvr_db,value,16);
			}
			else if(memcmp(name,"listen_threads",15) == 0)
			{
				(*ctx)->config->listen_threads = atoi(value);
			}
			else if(memcmp(name,"use_cmnt",8) == 0)
			{
				(*ctx)->config->use_cmnt = atoi(value);
			}
			else if(memcmp(name,"vrsn",4) == 0)
			{
				strncpy((*ctx)->config->vrsn,value,100);
				strncat((*ctx)->config->vrsn,"\n",2);
			}
			else if(memcmp(name,"cmnt",4) == 0)
			{
				strncpy((*ctx)->config->cmnt,value,200);
				strncat((*ctx)->config->cmnt,"\n\n",4);
			}
			else if(memcmp(name,"adm_email",9) == 0)
			{
				strncpy((*ctx)->config->adm_email,value,100);
			}
			else if(memcmp(name,"sync_email",10) == 0)
			{
				strncpy((*ctx)->config->sync_email,value,100);
			}
			else if(memcmp(name,"acpt_v2",7) == 0)
			{
				(*ctx)->config->acpt_v2 = atoi(value);
			}
			else if(memcmp(name,"acpt_v3",7) == 0)
			{
				(*ctx)->config->acpt_v3 = atoi(value);
			}
			else if(memcmp(name,"max_qry",7) == 0)
			{
				(*ctx)->config->max_qry = atoi(value);
			}
			else if(memcmp(name,"max_ret",7) == 0)
			{
				(*ctx)->config->max_ret = atoi(value);
			}
			else if(memcmp(name,"key_ret",7) == 0)
			{
				(*ctx)->config->key_ret = atoi(value);
			}
			else if(memcmp(name,"dup_acpt",7) == 0)
			{
				(*ctx)->config->dup_acpt = atoi(value);
			}
			else if(memcmp(name,"biglumber",9) == 0)
			{
				(*ctx)->config->biglumber = atoi(value);
			}
			else if(memcmp(name,"aonf",4) == 0)
			{
				(*ctx)->config->aonf = atoi(value);
			}
			else if(memcmp(name,"image_pkts",4) == 0)
			{
				(*ctx)->config->image_pkts = atoi(value);
			}
			else if(memcmp(name,"pid_dir",7) == 0)
			{
				snprintf((*ctx)->config->pid_file,255,"%s/cksd.pid",value);
			}
			else if(memcmp(name,"err_log",7) == 0)
			{
				strncpy((*ctx)->config->err_log,value,200);
			}
			else if(memcmp(name,"mail_err_log",12) == 0)
			{
				strncpy((*ctx)->config->mail_err_log,value,200);
			}
			else if(memcmp(name,"data_log",8) == 0)
			{
        			strncpy((*ctx)->config->data_log,value,200);
        		}
			else if(memcmp(name,"cache",5) == 0)
			{
				if(load_cache == 1)
				{
					rslt = add_cache_node(&((*ctx)->cache),value);
					if(rslt == -1)
					{
						fprintf(stderr,"Failed to add cache node for %s.\n",value);
					}
				}
			}
			else
			{
				fprintf(stderr,_("cks_config:  Rejected config var: %s\n"),name);
			}
		}
	}
	fclose(conf_file);

	return 0;
}


int add_cache_node(struct d_linked_list **cache,char *fp)
{
	struct d_linked_list *new_list = NULL;
	int rslt = 0;


	new_list = new_dll_node(fp,(strlen(fp)+1),NULL,0);
	if(new_list == NULL)
	{
		fprintf(stderr,"Failed to create new node.\n");

		return -1;
	}

	rslt = add_dll_item(&(*cache),new_list);
	if(rslt == -1)
	{
		fprintf(stderr,"Failed to add node to cache.\n");

		return -1;
	}

	return 0;
}
