/* cks_reconcile.c - Synchronization Reconciliation main source file
 * Copyright (C) 2001-2005 CryptNET, V. Alex Brennen (VAB)
 *
 * This file is part of the CryptNET OpenPGP Public Key Server (cks).
 *
 * cks is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * cks is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "cks_reconcile.h"


int main(int argc, char *argv[])
{
	struct  cks_ctx *ctx = NULL;

	PGconn          *conn = NULL;
	PGresult        *result = NULL;

	unsigned	long	to_export = 0;
	unsigned 	long	i = 0;
	
	int	rslt = 0;

	unsigned int arg = 0;
	unsigned int verbose = 0;

	struct	d_linked_list	*local_keys = NULL;
	struct	d_linked_list	*foreign_keys = NULL;
	struct	d_linked_list	*rec_keys  = NULL;

	unsigned char *rec_host = NULL;


	ctx = (struct cks_ctx *)malloc(sizeof(struct cks_ctx));
	if(ctx == NULL)
	{
		fprintf(stderr,_("cks_reconcile:  Fatal Error:  Malloc call failed: Out of Memorry.\n"));

		return -1;
	}
	ctx->config = (struct cks_config *)malloc(sizeof(struct cks_config));
	if(ctx->config == NULL)
	{
		fprintf(stderr,_("cks_reconcile: Fatal Error:  Malloc Call Failed: Out of memroy.\n"));

		return -1;
	}
	rslt = init_config(&ctx,0);
	if(rslt == -1)
	{
		fprintf(stderr,_("cks_reconcile:  Non-Fatal Error: Failed to read config.\n"));
		fprintf(stderr,_("cks_reconcile:  Using default configuration information.\n"));
	}

	if(argc > 0)
	{
		for(arg=1;arg<argc;arg++)
		{
			if(argv[arg][0] == '-')
			{
				if(argv[arg][1] == '-')
				{
					if(strstr(argv[arg],"help") != NULL)
					{
						printf("Usage: cks_reconcile\n");
						printf("	-v Verbose Mode\n");
						printf("	-h This Help Text\n");
						printf("	--help This Help Text\n");
						printf("	--version Display Version Information\n");
						printf("\n");
						free_context(ctx);

						return 0;
					}
					else if(strstr(argv[arg],"version") != NULL)
					{
						printf("CKS Version %s\n", ctx->config->cks_version);
						free_context(ctx);

						return 0;
					}
				}
				else if(argv[arg][1] == 'v')
				{
					verbose = 1;
				}
				else
				{
					printf("Usage: cks_reconcile\n");
					printf("	-v Verbose Mode\n");
					printf("	-h This Help Text\n");
					printf("	--help This Help Text\n");
					printf("	--version Display Version Information\n");
					printf("\n");
					free_context(ctx);

					return 0;
				}
			}
		}
	}
	rslt = build_remote_key_list(ctx->config,&foreign_keys,/*rec_host*/ "gnv.us.ks.cryptnet.net");
	if(rslt == -1)
	{
		fprintf(stderr,"build_remote_key_list failed.\n");
		free_context(ctx);

		return -1;
	}

	/* Make the DB Connection. */
	conn = db_connect(ctx->config);
	if(conn == NULL)
	{
		fprintf(stderr,_("cks_reconcile: Failed to connect to postgres database"));
		fprintf(stderr,_("cks_reconcile: Connection to database failed.\n"));
		fprintf(stderr,"cks_reconcile: %s", PQerrorMessage(conn));
		free_context(ctx);

		return -1;
	}

	result = PQexec(conn,"select fp from cks_fp_key_table\n");
	if(PQresultStatus(result) != PGRES_TUPLES_OK)
	{
		fprintf(stderr,_("cks_reconcile:  Failed to return tuples.\n"));
		fprintf(stderr,_("cks_reconcile:  Offending Query: select fp from cks_fp_key_table\n"));
		PQclear(result);
		db_disconnect(conn);
		free_context(ctx);

		return -1;
	}

	to_export = PQntuples(result);
	if(to_export == 0)
	{
		fprintf(stderr, _("cks_reconcile:  No Records Found.\n"));
		fprintf(stderr,_("cks_reconcile:  Offending Query: select fp from cks_fp_key_table\n"));
		PQclear(result);
		db_disconnect(conn);
		free_context(ctx);

		return -1;
	}

	/* build a linked list of all of the keys in the database */
	for(i=0;i<to_export;i++)
	{
		PQgetvalue(result,i,0);
		/* local_keys; */
	}

	/* perform the actual sync with the remote keyserver */
	rslt = build_remote_key_list(ctx->config,&foreign_keys,/*rec_host*/ "gnv.us.ks.cryptnet.net");
	if(rslt == -1)
	{
		fprintf(stderr,"build_remote_key_list failed.\n");
		PQclear(result);
		db_disconnect(conn);
		free_context(ctx);

		return -1;
	}
	rslt = perform_reconciliation(ctx->config,local_keys,foreign_keys,/*rec_host*/ "gnv.us.ks.cryptnet.net");
	if(rslt == -1)
	{
		fprintf(stderr,"perform_reconciliation failed.\n");
		PQclear(result);
		db_disconnect(conn);
		free_context(ctx);

		return -1;
	}
	rslt = transmit_reconciliation_keys(ctx->config,rec_keys,rec_host);
	if(rslt == -1)
	{
		fprintf(stderr,"transmit_reconciliation_keys failed.\n");
		PQclear(result);
		db_disconnect(conn);
		free_context(ctx);

		return -1;
	}

	/* Finish.  Clean up memory and exit. */
	PQclear(result);
	db_disconnect(conn);

	free_context(ctx);

	return 0;
}


int build_remote_key_list(struct cks_config *config,struct d_linked_list **remote_keys,
	char *foreign_host)
{
	/* We should pass it a null linked list here and return a pointer to mallocd memory.  We can
	   then walk the linked list in the program and free it when we are finished.  With a usage
	   of 40 bytes per key and an estimation of 2million keys, we're looking at 80million bytes
	   of ~8 megabytes.  While that is a tremendous amount, it is well with the acceptable range
	   of usage for a program which is infrequently run. */

	/* Socket Vars */
	int	sockfd = 0;
	struct	hostent	*he = NULL;
	struct	sockaddr_in	target_addr;

	char	buff[256];
	int z = 0;

	memset(buff,0x00,255);
	/* open a socket to the server */
	if((he = gethostbyname(foreign_host)) == NULL)
	{
		fprintf(stderr,"Failed to get host by name.\n");

		return -1;
	}

	if((sockfd = socket(AF_INET,SOCK_STREAM,0)) == -1)
	{
		fprintf(stderr,"Failed to create socket.\n");

		return -1;
	}

	target_addr.sin_family = AF_INET;
	target_addr.sin_port = htons(11371);
	target_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(target_addr.sin_zero,0,sizeof(target_addr.sin_zero));

	if(connect(sockfd,(struct sockaddr *)&target_addr,
		sizeof(struct sockaddr)) == -1)
	{
		fprintf(stderr,"Failed to open socket.\n");

		return -1;
	}

	write_line_to_socket(sockfd,"GET /pks/lookup?op=rec&search=NULL HTTP/1.0\r\n\r\n");

	do
	{
		z = read_line(sockfd,buff,255);
		printf("%s\n",buff);
	} while((z != 0) && (strncasecmp("Content-Length",buff,14) != 0));

	/* free mem, close sock, and rtrn */
	shutdown(sockfd,2);

	return 0;
}


int perform_reconciliation(struct cks_config *config, struct d_linked_list *list_local,
	struct d_linked_list *list_foreign, unsigned char *foreign_host)
{
	/* This needs to be moved to cks_common or some other place so that it can be called by cksd.
	   We want cksd to open a stocket to the sync target cksd and dump any keys not listed.
	 */
	struct d_linked_list *walk_local = NULL;
	struct d_linked_list *walk_foreign = NULL;

	int found = 0;
	int rslt = 0;


	walk_local = get_first_dll_node(list_local);
	while(walk_local != NULL)
	{
		found = 0;

		walk_foreign = get_first_dll_node(list_foreign);
		while(walk_foreign != NULL)
		{
			if(strcmp(walk_local->name,walk_foreign->name) == 0)
			{
				found = 1;
				break;
			}
			walk_foreign = walk_foreign->next;
		}
		if(found == 0)
		{
			printf("Requesting:  %s\n", (char *)(walk_local->name));
			rslt = fetch_key(config,walk_local->name,foreign_host);
			if(rslt == -1)
			{
				fprintf(stderr,"Failed to fetch key %s\n",(char *)(walk_local->name));
			}
		}
	 	walk_local = walk_local->next;
	}

	return 0;
}


int transmit_reconciliation_keys(struct cks_config *config, struct d_linked_list *rec_keys,
	char *foreign_host)
{
	/* Socket Vars */
	int	sockfd = 0;
	struct	hostent	*he = NULL;
	struct	sockaddr_in	target_addr;

	struct d_linked_list *walk_key = NULL;


	/* Open a socket to the remote server */
	if((he = gethostbyname(foreign_host)) == NULL)
	{
		fprintf(stderr,"Failed to get host by name.\n");

		return -1;
	}

	if((sockfd = socket(AF_INET,SOCK_STREAM,0)) == -1)
	{
		fprintf(stderr,"Failed to create socket.\n");

		return -1;
	}

	target_addr.sin_family = AF_INET;
	target_addr.sin_port = htons(11371);
	target_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(target_addr.sin_zero,0,sizeof(target_addr.sin_zero));

	if(connect(sockfd,(struct sockaddr *)&target_addr,
		sizeof(struct sockaddr)) == -1)
	{
		fprintf(stderr,"Failed to open socket.\n");

		return -1;
	}

	/* Walk the key list and transmit */
	walk_key = get_first_dll_node(rec_keys);
	while(walk_key != NULL)
	{
		write_line_to_socket(sockfd,"");

		walk_key = walk_key->next;
	}

	/* free mem, close sock, and rtrn */
	shutdown(sockfd,2);

	return 0;
}


int fetch_key(struct cks_config *config, char *fp, char *foreign_host)
{
	int	sockfd = 0;
	struct	hostent	*he = NULL;
	struct	sockaddr_in	target_addr;

	int	z = 0;
	char	buff[1024];
	

	memset(buff,0x00,1024);
	/* open a socket to the server */
	if((he = gethostbyname(foreign_host)) == NULL)
	{
		fprintf(stderr,"Failed to get host by name.\n");

		return -1;
	}

	if((sockfd = socket(AF_INET,SOCK_STREAM,0)) == -1)
	{
		fprintf(stderr,"Failed to create socket.\n");

		return -1;
	}

	target_addr.sin_family = AF_INET;
	target_addr.sin_port = htons(11371);
	target_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(target_addr.sin_zero,0,sizeof(target_addr.sin_zero));

	if(connect(sockfd,(struct sockaddr *)&target_addr,
		sizeof(struct sockaddr)) == -1)
	{
		fprintf(stderr,"Failed to open socket.\n");

		return -1;
	}

	/* request the key */
	write_line_to_socket(sockfd,"");
	write_line_to_socket(sockfd,"");
	write_line_to_socket(sockfd,"");

	/* read it into a buffer */
	do
	{
		z = read_line(sockfd,buff,64);
	} while ((z != 0));

	/* import it */

	/* free mem, close sock, and rtrn */
	shutdown(sockfd,2);

	return 0;
}
