/* datastructures.c - Generic Datastructures functions file
 * Copyright (C) 2001-2005 CryptNET, V. Alex Brennen (VAB)
 *
 * This file is part of the CryptNET OpenPGP Public Key Server (cks).
 *
 * cks is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * cks is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "datastructures.h"

/*
 * Navigational Code For Data Structures 
 *
 */

struct name_value_pair_dllst* new_nvp_dll_node(void *name, size_t name_size,
						void *value, size_t value_size)
{
	struct name_value_pair_dllst *new_node = NULL;

	new_node = (struct name_value_pair_dllst *)malloc(sizeof(struct name_value_pair_dllst));
	if(new_node != NULL)
	{
		new_node->next = NULL;
		new_node->prev = NULL;
		new_node->name = NULL;
		new_node->value = NULL;
		new_node->name = malloc(name_size+2);
		if(new_node->name != NULL)
		{
			strncpy(new_node->name,name,name_size+1);
		}
		else
		{
			free(new_node);
			new_node = NULL;
		}
		if(value != NULL)
		{
			new_node->value = malloc(value_size+2);
			if(new_node->value != NULL)
			{
				strncpy(new_node->value,value,value_size+1);
			}
			else
			{
				free(new_node->name);
				free(new_node);
				new_node = NULL;
			}
		}
	}

	return new_node;
}


struct name_value_pair_dllst *get_first_pair(struct name_value_pair_dllst *the_list)
{
	if(the_list != NULL)
	{
		while(the_list->prev != NULL)
		{
			the_list = the_list->prev;
		}
	}

	return the_list;
}


struct name_value_pair_dllst *get_last_pair(struct name_value_pair_dllst *the_list)
{
	if(the_list != NULL)
	{
		while(the_list->next != NULL)
		{
			the_list = the_list->next;
		}
	}

	return the_list;
}


char *get_value(struct name_value_pair_dllst *the_list,char *name)
{
	if(the_list == NULL)
	{
		return NULL;
	}

	/* Rewind to the beginning */
	the_list = get_first_pair(the_list);

	while(the_list != NULL)
	{
		if(strcmp(the_list->name,name) == 0)
		{
			return the_list->value;
		}
		else
		{
			the_list = the_list->next;
		}
	}

	return NULL;
}


/* d_linked_list */
struct d_linked_list * new_dll_node(void *name, size_t name_size, void *value, size_t value_size)
{
	struct d_linked_list *new_node = NULL;


	new_node = (struct d_linked_list *)malloc(sizeof(struct d_linked_list));
	if(new_node != NULL)
	{
		new_node->next = NULL;
		new_node->prev = NULL;
		new_node->name = NULL;
		new_node->value = NULL;
		new_node->name = malloc(name_size);
		if(new_node->name != NULL)
		{
			memcpy(new_node->name,name,name_size);
		}
		else
		{
			free(new_node);
			new_node = NULL;
		}
		if(value != NULL)
		{
			new_node->value = malloc(value_size);
			if(new_node->value != NULL)
			{
				memcpy(new_node->value,value,value_size);
			}
			else
			{
				free(new_node->name);
				free(new_node);
				new_node = NULL;
			}
		}
	}

	return new_node;
}


int add_dllst_item(struct name_value_pair_dllst **the_list, struct name_value_pair_dllst *new_node)
{
	int rslt = 0;
	struct name_value_pair_dllst *local_list;


	if(the_list != NULL)
	{
		local_list = get_last_pair(*the_list);
		if(local_list == NULL)
		{
			local_list = new_node;
		}
		else
		{
			local_list->next = new_node;
			new_node->prev = local_list;
		}
		*the_list = local_list;
	}
	else
	{
		rslt = -1;
	}

	return rslt;
}


int add_dll_item(struct d_linked_list **the_list, struct d_linked_list *new_node)
{
	int rslt = 0;
	struct d_linked_list *local_list = NULL;

	if(the_list != NULL)
	{
		local_list = get_last_dll_node(*the_list);
		if(local_list == NULL)
		{
			local_list = new_node;
		}
		else
		{
			local_list->next = new_node;
			new_node->prev = local_list;
		}
		*the_list = local_list;
	}
	else
	{
		rslt = -1;
	}

	return rslt;
}


struct d_linked_list * get_first_dll_node(struct d_linked_list *the_list)
{
	if(the_list != NULL)
	{
		while(the_list->prev != NULL)
		{
			the_list = the_list->prev;
		}
	}

	return the_list;
}


struct d_linked_list * get_last_dll_node(struct d_linked_list *the_list)
{
	if(the_list != NULL)
	{
		while(the_list->next != NULL)
		{
			the_list = the_list->next;
		}
	}

	return the_list;
}


void free_dll(struct d_linked_list **the_list)
{
	struct d_linked_list *walk_list = NULL;
	struct d_linked_list *next_node = NULL;

	if(the_list != NULL)
	{
		walk_list = get_first_dll_node(*the_list);
		while(walk_list != NULL)
		{
			next_node = walk_list->next;
			if(walk_list->name != NULL)
				free(walk_list->name);
			if(walk_list->value != NULL)
				free(walk_list->value);
			free(walk_list);
			walk_list = next_node;
		}
	}

	return;
}
/* /d_linked_list */


void free_name_value_pair_dllst(struct name_value_pair_dllst *the_list)
{
	struct name_value_pair_dllst *walk_pair = NULL;
	struct name_value_pair_dllst *next_pair = NULL;


	if(the_list != NULL)
	{
		walk_pair = (struct name_value_pair_dllst *)get_first_pair(the_list);
		while(walk_pair != NULL)
		{
			next_pair = walk_pair->next;
			if(walk_pair->name != NULL)
			{
				free(walk_pair->name);
			}
			if(walk_pair->value != NULL)
			{
				free(walk_pair->value);
			}
			free(walk_pair);
			walk_pair = next_pair;
		}
	}

	return;
}
